@@ -8,9 +8,11 @@ import sys
 from typing import Awaitable
 from typing import Callable
 from typing import List
+from typing import Literal
 from typing import Optional
 from typing import Set
 
+from aiohttp import ClientSession
 from aiohttp import web
 from aiohttp.web import Request
 from aiohttp.web import middleware
@@ -25,8 +27,9 @@ from .snapshot import snapshot
 from .trace import Trace
 from .trace import TraceMap
 from .trace import decode_v04
+from .trace import decode_v05
 from .trace import pprint_trace
-from .trace import v04TraceChunk
+from .trace import v04TracePayload
 from .trace_checks import CheckMetaTracerVersionHeader
 from .trace_checks import CheckTraceContentLength
 from .trace_checks import CheckTraceCountHeader
@@ -170,12 +173,24 @@ class Agent:
                         tracemap[trace_id].append(span)
         return list(tracemap.values())
 
-    async def _decode_v04_traces(self, request: Request) -> v04TraceChunk:
+    async def _decode_v04_traces(self, request: Request) -> v04TracePayload:
         content_type = request.content_type
         raw_data = await request.read()
         return decode_v04(content_type, raw_data)
 
+    async def _decode_v05_traces(self, request: Request) -> v04TracePayload:
+        raw_data = await request.read()
+        return decode_v05(raw_data)
+
     async def handle_v04_traces(self, request: Request) -> web.Response:
+        return await self._handle_traces(request, version="v0.4")
+
+    async def handle_v05_traces(self, request: Request) -> web.Response:
+        return await self._handle_traces(request, version="v0.5")
+
+    async def _handle_traces(
+        self, request: Request, version: Literal["v0.4", "v0.5"]
+    ) -> web.Response:
         self._requests.append(request)
         checks: Checks = request.app["checks"]
 
@@ -186,7 +201,10 @@ class Agent:
                 "trace_content_length",
                 content_length=int(request.headers["Content-Length"]),
             )
-            traces = await self._decode_v04_traces(request)
+            if version == "v0.4":
+                traces = await self._decode_v04_traces(request)
+            elif version == "v0.5":
+                traces = await self._decode_v05_traces(request)
             log.info("received trace payload with %r trace chunks", len(traces))
             for i, trace in enumerate(traces):
                 try:
@@ -208,12 +226,23 @@ class Agent:
                     num_traces=len(traces),
                 )
 
+        agent_url = request.app["agent_url"]
+        if agent_url:
+            log.info("Forwarding request to agent at %r", agent_url)
+            async with ClientSession() as session:
+                async with session.post(
+                    f"{agent_url}/v0.4/traces",
+                    headers=request.headers,
+                    data=await request.read(),
+                ) as resp:
+                    assert resp.status == 200
+                    data = await resp.json()
+                    log.info("Got response %r from agent", data)
+                    return web.json_response(data=data)
+
         # TODO: implement sampling logic
         return web.json_response(data={"rate_by_service": {}})
 
-    async def handle_v05(self, request: Request) -> web.Response:
-        raise NotImplementedError
-
     async def handle_session_start(self, request: Request) -> web.Response:
         self._requests.append(request)
         return web.HTTPOk()
@@ -339,6 +368,7 @@ def make_app(
     snapshot_dir: str,
     snapshot_ci_mode: bool,
     snapshot_ignored_attrs: List[str],
+    agent_url: str,
 ) -> web.Application:
     agent = Agent()
     app = web.Application(
@@ -352,7 +382,8 @@ def make_app(
         [
             web.post("/v0.4/traces", agent.handle_v04_traces),
             web.put("/v0.4/traces", agent.handle_v04_traces),
-            web.put("/v0.5/traces", agent.handle_v05),
+            web.post("/v0.5/traces", agent.handle_v05_traces),
+            web.put("/v0.5/traces", agent.handle_v05_traces),
             web.get("/test/session/start", agent.handle_session_start),
             web.get("/test/session/clear", agent.handle_session_clear),
             web.get("/test/session/snapshot", agent.handle_snapshot),
@@ -375,6 +406,7 @@ def make_app(
     app["snapshot_ci_mode"] = snapshot_ci_mode
     app["log_span_fmt"] = log_span_fmt
     app["snapshot_ignored_attrs"] = snapshot_ignored_attrs
+    app["agent_url"] = agent_url
     return app
 
 
@@ -446,6 +478,17 @@ def main(args: Optional[List[str]] = None) -> None:
             "All span attributes are available."
         ),
     )
+    parser.add_argument(
+        "--agent-url",
+        type=str,
+        default=os.environ.get(
+            "DD_TRACE_AGENT_URL", os.environ.get("DD_AGENT_URL", "")
+        ),
+        help=(
+            "Datadog agent URL. If provided, any received data will be forwarded "
+            "to the agent."
+        ),
+    )
     parsed_args = parser.parse_args(args=args)
     logging.basicConfig(level=parsed_args.log_level)
 
@@ -466,6 +509,7 @@ def main(args: Optional[List[str]] = None) -> None:
         snapshot_dir=parsed_args.snapshot_dir,
         snapshot_ci_mode=parsed_args.snapshot_ci_mode,
         snapshot_ignored_attrs=parsed_args.snapshot_ignored_attrs,
+        agent_url=parsed_args.agent_url,
     )
     web.run_app(app, port=parsed_args.port)
 
